package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
)

// ExampleBufio
func main() {
	//path := "/etc/hosts"
	path := "a.txt"

	// 每次扫描一行
	// 打开文件
	file, err := os.Open(path)
	// 结束时关闭文件
	defer file.Close()
	if err != nil {
		fmt.Println(err.Error())
		return
	} else {
		fmt.Printf("%T\n", file)

		// Scanner
		sc := bufio.NewScanner(file)
		for sc.Scan() {
			// 输出一行的内容
			fmt.Println(sc.Text())
		}
	}

	// 每次读指定字符
	file, err = os.Open(path)
	// 结束时关闭文件
	defer file.Close()
	if err != nil {
		fmt.Println(err.Error())
		return
	} else {
		// Reader
		reader := bufio.NewReader(file)
		bs := make([]byte, 10)
		for {
			n, err := reader.Read(bs)
			fmt.Println(n, err)
			if err != nil || err == io.EOF {
				fmt.Println(err.Error())
				break
			} else {
				fmt.Println(string(bs[:n]))
			}
		}
	}

	// 读一行
	file, err = os.Open(path)
	// 结束时关闭文件
	defer file.Close()
	if err != nil {
		fmt.Println(err.Error())
		return
	} else {
		// ReadLine
		reader := bufio.NewReader(file)
		for {
			bs, isPrefix, err := reader.ReadLine()
			fmt.Println(bs, isPrefix, err)
			if err != nil || err == io.EOF {
				fmt.Println(err.Error())
				break
			} else {
				fmt.Println(string(bs))
			}
		}
	}

	// 按指定分隔符读
	file, err = os.Open(path)
	// 结束时关闭文件
	defer file.Close()
	if err != nil {
		fmt.Println(err.Error())
		return
	} else {
		// ReadSlice
		fmt.Println("ReadSlice/ReadString")
		reader := bufio.NewReader(file)
		for {
			bs, err := reader.ReadSlice('\n')
			//bs, err := reader.ReadString('\n')
			if err != nil || err == io.EOF {
				fmt.Println(err.Error())
				break
			} else {
				fmt.Println(string(bs))
			}
		}
	}

	// 带缓冲 io 的 write
	file, err = os.Create(path)
	// 结束时关闭文件
	defer file.Close()
	if err != nil {
		fmt.Println(err.Error())
		return
	} else {
		writer := bufio.NewWriter(file)
		writer.Write([]byte("hello\n"))
		writer.WriteString("bufio\n")
		writer.Flush()
	}
}
